"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
function _fs() {
  const data = _interopRequireDefault(require("fs"));
  _fs = function _fs() {
    return data;
  };
  return data;
}
function _path() {
  const data = _interopRequireDefault(require("path"));
  _path = function _path() {
    return data;
  };
  return data;
}
var globals = _interopRequireWildcard(require("./globals"));
var _buildcontents = _interopRequireDefault(require("./buildcontents"));
var _generateoutput = _interopRequireDefault(require("./generateoutput"));
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && Object.prototype.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function writeToFile(path, data) {
  return new Promise(function (resolve, reject) {
    try {
      var fd = _fs().default.openSync(path, 'w'),
        buffer = new Buffer(data.join('\n'), 'utf8');
      _fs().default.writeSync(fd, buffer, 0, buffer.length, 0);
      _fs().default.closeSync(fd);
      resolve(null);
    } catch (err) {
      console.log('Could not write to file: ' + JSON.stringify(err, null, 4));
      reject(err);
    }
  });
}
/**
 * Uses the config to go through all of the framework *.less files in the
 * proper order and compresses them into a single file for packaging.
 *
 * @param config The configuration for compressing the files.
 * when the task is complete.
 */
function compress(config) {
  globals.initialize(config);
  return new Promise(function (resolve, reject) {
    var src = _path().default.resolve(globals.config.src),
      writeFile = config && config.writeFile,
      writers = globals.writers,
      output = globals.output,
      dest = globals.config.dest,
      clear = globals.clear,
      version = globals.config.version,
      license = globals.config.license;
    // Goes through each file in the dest files and makes sure they have a
    // .less extension.
    if (writeFile) {
      dest.forEach((outFile, index) => {
        var end = outFile.lastIndexOf('.');
        dest[index] = outFile.substring(0, end > -1 ? end : undefined) + '.less';
      });
    }
    // Reads the src file, builds the contents for each
    // file in the proper order, and generates the output file.
    _fs().default.readFile(src, 'utf8', (err, data) => {
      if (err) {
        return reject(err);
      }
      var splitLines = data.split(/\r\n|\n/);
      splitLines[0] = splitLines[0].trim();
      (0, _buildcontents.default)(splitLines, src);
      // generate the output
      (0, _generateoutput.default)();
      // If a license file is specified, we want to prepend it to the output.
      if (!!license) {
        var licenseFile = _path().default.resolve(license),
          licenseData = _fs().default.readFileSync(licenseFile, 'utf8'),
          lines = licenseData.split(/\r\n|\n/),
          regex = /(.*)v\d+\.\d+\.\d+(.*)/;
        // If a version is specified, we want to go through and find where
        // the version is specified in the license, then replace it with the
        // passed-in version.
        if (!!version) {
          lines.some((line, index) => {
            if (regex.test(line)) {
              lines[index] = line.replace(regex, '$1v' + version + '$2');
              return true;
            }
            return false;
          });
        }
        // Add the lines as a comment block
        lines.forEach((line, index) => {
          lines[index] = ' * ' + line;
        });
        lines.unshift('/**');
        lines.push(' */');
        output.unshift(lines.join('\n'));
      }
      // Make sure the file ends in a new line.
      if (!!output[output.length - 1].trim()) {
        output.push('');
      }
      // Go through each destination file and make sure we can
      // write a file to the location, making new directories
      // if necessary. Then write the output to each destination.
      if (writeFile) {
        dest.forEach(destFile => {
          var destPath = _path().default.resolve(destFile),
            split = _path().default.resolve(destPath, '..').split(/\/|\\/),
            concat = '';
          if (/[a-zA-Z]:/.test(split[0])) {
            split.shift();
            split.unshift('/' + split.shift());
          }
          split.forEach(val => {
            concat += val + '/';
            try {
              _fs().default.mkdirSync(concat);
            } catch (err) {}
          });
          writeToFile(destPath, output).then(() => resolve(output.join('\n'))).catch(error => reject(error));
        });
      }
      clear();
      resolve(output.join('\n'));
    });
  });
}
var _default = exports.default = compress;