import Writer from './writer';
export interface IConfig {
    /**
     * The path to a LESS file that imports all the desired files
     * in the order you wish them to be bundled in.
     */
    src: string;
    /**
     * An array of destination file paths. Once the framework
     * is built, it will be output to these paths.
     */
    dest?: Array<string>;
    /**
     * Boolean flag to indicate that if we want the compiled css into dest file or not
     */
    writeFile?: boolean;
    /**
     * The version number used in conjunction with the license.
     */
    version?: string;
    /**
     * The path to the license file to be added to the build as a
     * comment. If a version is specified, the v.0.0.0 in the
     * license will be replaced with the version.
     */
    license?: string;
    /**
     * Clear global cache var
     */
    clear?: () => void;
}
export interface IObject<T> {
    [x: string]: T;
}
/**
 * Creates the config.
 *
 * @param cfg The root config.
 */
export declare function initialize(cfg: IConfig): IConfig;
/**
 * Clear cache
 */
export declare function clear(): void;
export declare var config: IConfig, writers: Array<Writer>, output: Array<string>, imports: IObject<boolean>, hrefRegex: RegExp, stringLiteralRegex: RegExp, lessFileRegex: RegExp, cssFileRegex: RegExp;
