"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clear = clear;
exports.imports = exports.hrefRegex = exports.cssFileRegex = exports.config = void 0;
exports.initialize = initialize;
exports.writers = exports.stringLiteralRegex = exports.output = exports.lessFileRegex = void 0;
function isString(obj) {
  return typeof obj === 'string';
}
function isArray(obj) {
  return Array.isArray(obj);
}
function validate(config) {
  var errors = [];
  if (!(isString(config.src) && lessFileRegex.test(config.src))) {
    errors.push('Error: src config property must be a string path locating the LESS file for the bundle');
  }
  if (config.writeFile && !isArray(config.dest)) {
    errors.push('Error: dest config property must be a string or array of strings designating the output LESS file(s).');
  }
  return errors;
}
/**
 * Creates the config.
 *
 * @param cfg The root config.
 */
function initialize(cfg) {
  if (!cfg) {
    throw new Error('No config specified');
  }
  exports.config = config = cfg;
  if (typeof cfg.dest === 'string') {
    config.dest = [cfg.dest];
  }
  var errors = validate(config);
  if (errors.length > 0) {
    errors.forEach(error => {
      console.log(error);
    });
    throw new Error('Invalid config');
  }
  return config;
}
/**
 * Clear cache
 */
function clear() {
  exports.writers = writers = [];
  exports.output = output = [];
  exports.imports = imports = {};
}
var config,
  writers = exports.writers = [],
  output = exports.output = [],
  // hashes the import statements
  imports = exports.imports = {},
  // Finds all the <script src="" /> tags
  hrefRegex = exports.hrefRegex = /href=("[^"]*)/,
  // Finds the start comment Node
  // startRegex = /<!--\s*less-bundle-start/,
  // Finds the end comment Node
  // endRegex = /<!--\s*less-bundle-end/,
  // Finds the string literal in a string
  stringLiteralRegex = exports.stringLiteralRegex = /.*(?:'|")(.*)(?:'|").*/,
  // Tests for less file
  lessFileRegex = exports.lessFileRegex = /.less$/,
  // Tests for css file
  cssFileRegex = exports.cssFileRegex = /.css$/;