"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
const pluginName = 'postcss-ant-prefix';
const plugin = options => {
  if (!options) {
    throw new Error(`[${pluginName}]: you must provide options to use this plugin`);
  }
  const prefixConfig = options.prefix;
  const prefixRules = Object.keys(prefixConfig).map(k => ({
    key: k,
    reg: new RegExp(`\\b${k}`, 'g'),
    value: prefixConfig[k]
  }));
  const excludes = options.excludes;
  const includes = options.includes;
  const test = options.test;
  const prefixKeys = Object.keys(prefixConfig);
  const isMatch = file => {
    if (!file) {
      return false;
    }
    const normalFilePath = file.replace(/\\/g, '/');
    if (test) {
      return test(normalFilePath);
    }
    if (!excludes && !includes) {
      return true;
    }
    if (excludes && includes) {
      return excludes.every(v => !v.test(normalFilePath)) && includes.some(v => v.test(normalFilePath));
    }
    if (excludes) {
      return excludes.every(v => !v.test(normalFilePath));
    }
    return includes.some(v => v.test(normalFilePath));
  };
  return {
    postcssPlugin: pluginName,
    Once(root) {
      const file = root.source.input.file;
      if (!isMatch(file)) {
        return;
      }
      root.walk(node => {
        if (node.type === 'rule') {
          const rule = node;
          prefixRules.forEach(({
            reg,
            value
          }) => {
            rule.selector = rule.selector.replace(reg, value);
          });
        }
      });
    }
  };
};
plugin.postcss = true;
var _default = exports.default = plugin;