"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "default", {
  enumerable: true,
  get: function get() {
    return _transformToLess.default;
  }
});
exports.transformCombineLess2css = transformCombineLess2css;
function _postcss() {
  const data = _interopRequireDefault(require("postcss"));
  _postcss = function _postcss() {
    return data;
  };
  return data;
}
function _postcssLess() {
  const data = _interopRequireDefault(require("postcss-less"));
  _postcssLess = function _postcssLess() {
    return data;
  };
  return data;
}
function _less() {
  const data = _interopRequireDefault(require("less"));
  _less = function _less() {
    return data;
  };
  return data;
}
var _log = require("../log");
var _getUserConfig = require("./getUserConfig");
function _fs() {
  const data = require("fs");
  _fs = function _fs() {
    return data;
  };
  return data;
}
function _path() {
  const data = _interopRequireDefault(require("path"));
  _path = function _path() {
    return data;
  };
  return data;
}
function _cssnano() {
  const data = _interopRequireDefault(require("cssnano"));
  _cssnano = function _cssnano() {
    return data;
  };
  return data;
}
var _transformToLess = _interopRequireDefault(require("./transformToLess"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
function transformLessToLess1(_x, _x2) {
  return _transformLessToLess.apply(this, arguments);
}
function _transformLessToLess() {
  _transformLessToLess = _asyncToGenerator(function* (mainLessFile, config) {
    try {
      const colorless = compile({
        nodeModulesPath: _path().default.resolve(process.cwd(), './node_modules'),
        resourcePath: mainLessFile,
        contextPath: _path().default.dirname(mainLessFile)
      });
      const _getPreConfig = (0, _getUserConfig.getPreConfig)(config),
        postcssConfig = _getPreConfig.postcssConfig;
      const _yield$postcss$proces = yield (0, _postcss().default)(postcssConfig.postcssOptions.plugins).process(colorless, {
          syntax: _postcssLess().default,
          from: mainLessFile
        }),
        css = _yield$postcss$proces.css;
      return css;
    } catch (err) {
      (0, _log.error)(err);
    }
  });
  return _transformLessToLess.apply(this, arguments);
}
function transformCombineLess2css(_x3, _x4) {
  return _transformCombineLess2css.apply(this, arguments);
}
function _transformCombineLess2css() {
  _transformCombineLess2css = _asyncToGenerator(function* (mainLessFile, config) {
    try {
      const lessStr = yield transformLessToLess1(mainLessFile, config);
      const rs = yield _less().default.render(lessStr);
      const compress = config.compressStyle !== false;
      if (!compress) {
        return rs.css;
      }
      const minCssResult = yield (0, _postcss().default)([(0, _cssnano().default)({})]).process(rs.css, {
        from: mainLessFile
      });
      return minCssResult.css;
    } catch (e) {
      console.error(mainLessFile, e);
    }
  });
  return _transformCombineLess2css.apply(this, arguments);
}
const __LINE__ = '\r\n';
const __LINE_PATTERN_ = /[\r\n]+/;
/**
 * 合并多个less
 * @param {CombineLessOptions} options
 * @param imports
 * @param {string[]} globalImports
 * @param {Object.<string, string|((p:string)=>string)>} mappings
 * @returns {string}
 */
function compile(options, imports = false, globalImports = [], mappings = {}) {
  /**
   * @type {Object.<string, boolean>}
   */
  const cache = {};
  /**
   * @type {Object.<string, string[]>}
   */
  const varsCache = {};
  /**
   * @type {string[]}
   */
  const vars = [];
  /**
   * @param {CombineLessOptions} options
   * @returns {string}
   */
  function combineLess(options) {
    const resourcePath = options.resourcePath,
      contextPath = options.contextPath,
      nodeModulesPath = options.nodeModulesPath;
    const fileContent = (0, _fs().readFileSync)(resourcePath).toString();
    const directory = _path().default.dirname(resourcePath);
    if (!varsCache[resourcePath]) {
      varsCache[resourcePath] = [];
    }
    /**
     * @param {string} l
     * @returns {boolean}
     */
    const isVarEnd = l => /;$/.test(l);
    let skipLineCount = 0;
    /**
     * @param {string} line
     * @param {string[]} lines
     * @param {number} i
     * @returns {string|string[]}
     */
    const getLine = (line, lines, i) => {
      skipLineCount = 0;
      let tr = line.trim();
      if (/^$|^\/\//.test(tr)) {
        return '';
      }
      tr = tr.replace(/(?<=;\s*)\/\/.*$/gm, '').trim();
      if (!tr.startsWith('@import')) {
        // TODO: 按行切割有错误，如果属性值换行将产生异常
        if (/^@[\w-]+/.test(tr) && !/@(keyframes|media)/.test(tr)) {
          if (isVarEnd(tr)) {
            varsCache[resourcePath].push(tr);
            vars.push(tr);
            return '';
          }
          const exp = [tr];
          let nextIndex = i + 1;
          let nextTr = lines[nextIndex];
          while (!isVarEnd(nextTr)) {
            exp.push(nextTr);
            nextIndex++;
            nextTr = lines[nextIndex];
          }
          exp.push(nextTr);
          skipLineCount = nextIndex - i + 1;
          const er = exp.join('').replace(/[\r\n]+/g, '');
          varsCache[resourcePath].push(er);
          vars.push(er);
          return '';
        }
        return tr;
      }
      let arr = tr.match(/@import ["'](.*)["'];/);
      if (arr === null) {
        console.error(`less @import 路径解析异常: ${line} # ${options.resourcePath}`);
        return '';
      }
      let importPath = arr[1];
      if (!importPath.endsWith('.less')) {
        importPath += '.less';
      }
      const isFromNodeModules = /^~/.test(importPath);
      let normalizedImportPath = isFromNodeModules ? importPath.slice(1) : importPath;
      const newPath = _path().default.join(isFromNodeModules ? nodeModulesPath : directory, normalizedImportPath);
      const relPath = isFromNodeModules ? importPath : _path().default.relative(contextPath, newPath);
      const mapKey = relPath.replace(/[\/\\]+/g, '/');
      let impPath = mappings[mapKey];
      if (cache[newPath]) {
        // vars.push(...varsCache[newPath]);
        return '';
      }
      cache[newPath] = true;
      const mapped = typeof impPath !== 'undefined' && impPath !== null;
      if (!mapped) {
        return combineLess({
          contextPath,
          resourcePath: newPath,
          nodeModulesPath
        });
      }
      if (!imports) {
        return '';
      }
      if (isFromNodeModules) {
        if (typeof impPath === 'function') {
          impPath = impPath(importPath);
        }
        if (!impPath) {
          return '';
        }
        globalImports.push(`@import "/${impPath.replace(/^\//, '')}";`);
        return '';
      }
      if (typeof impPath === 'function') {
        impPath = impPath(importPath);
      }
      if (!impPath) {
        return '';
      }
      globalImports.push(`@import "${impPath}";`);
      return '';
    };
    const lines = fileContent.split(__LINE_PATTERN_);
    /**
     * @type {string[]}
     */
    const result = [];
    for (let i = 0; i < lines.length; i++) {
      const line = lines[i];
      const rr = getLine(line, lines, i);
      i += skipLineCount;
      if (!rr) {
        continue;
      }
      if (Array.isArray(rr)) {
        result.push(...rr);
      } else {
        result.push(rr);
      }
    }
    return result.filter(Boolean).join(__LINE__);
  }
  const rr = combineLess(options);
  const finalVars = [...new Set(vars.reverse())].reverse();
  return finalVars.concat(rr).join(__LINE__);
}