"use strict";

function _webpack() {
  const data = _interopRequireDefault(require("webpack"));
  _webpack = function _webpack() {
    return data;
  };
  return data;
}
var _compile = require("../utils/compile");
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function _chalk() {
    return data;
  };
  return data;
}
function _path() {
  const data = _interopRequireDefault(require("path"));
  _path = function _path() {
    return data;
  };
  return data;
}
var _task = require("../task");
var _log = require("../log");
var _getUserConfig = _interopRequireDefault(require("../common/getUserConfig"));
var _loadDotEnv = _interopRequireDefault(require("../common/loadDotEnv"));
var _BabelRegister = _interopRequireDefault(require("../utils/BabelRegister/BabelRegister"));
function _webpackBundleAnalyzer() {
  const data = require("webpack-bundle-analyzer");
  _webpackBundleAnalyzer = function _webpackBundleAnalyzer() {
    return data;
  };
  return data;
}
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function getAnalzerPlugin(port = 9078) {
  const options = {
    //  可以是`server`，`static`或`disabled`。
    //  在`server`模式下，分析器将启动HTTP服务器来显示软件包报告。
    //  在“静态”模式下，会生成带有报告的单个HTML文件。
    //  在`disabled`模式下，你可以使用这个插件来将`generateStatsFile`设置为`true`来生成Webpack Stats JSON文件。
    analyzerMode: 'server',
    //  将在“服务器”模式下使用的主机启动HTTP服务器。
    analyzerHost: '127.0.0.1',
    //  将在“服务器”模式下使用的端口启动HTTP服务器。
    analyzerPort: port,
    //  路径捆绑，将在`static`模式下生成的报告文件。
    //  相对于捆绑输出目录。
    reportFilename: 'report.html',
    //  模块大小默认显示在报告中。
    //  应该是`stat`，`parsed`或者`gzip`中的一个。
    //  有关更多信息，请参见“定义”一节。
    defaultSizes: 'parsed',
    //  在默认浏览器中自动打开报告
    openAnalyzer: true,
    //  如果为true，则Webpack Stats JSON文件将在bundle输出目录中生成
    generateStatsFile: false,
    //  如果`generateStatsFile`为`true`，将会生成Webpack Stats JSON文件的名字。
    //  相对于捆绑输出目录。
    statsFilename: 'stats.json',
    //  stats.toJson（）方法的选项。
    //  例如，您可以使用`source：false`选项排除统计文件中模块的来源。
    //  在这里查看更多选项：https：  //github.com/webpack/webpack/blob/webpack-1/lib/Stats.js#L21
    statsOptions: null,
    logLevel: 'info' // 日志级别。可以是'信息'，'警告'，'错误'或'沉默'。
  };
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-ignore
  return new (_webpackBundleAnalyzer().BundleAnalyzerPlugin)(options);
}
module.exports = function startWatchAns(port = 8998) {
  if (!port) {
    (0, _log.error)('无效的端口号');
    return;
  }
  const cwd = process.cwd();
  const basePath = _path().default.join(cwd, '.env');
  const localPath = `${basePath}.local`;
  (0, _loadDotEnv.default)(basePath);
  (0, _loadDotEnv.default)(localPath);
  (0, _log.log)('register babel before config parsing');
  const babelRegister = new _BabelRegister.default();
  babelRegister.register();
  const userConfig = (0, _getUserConfig.default)({
    cwd,
    env: 'development'
  });
  (0, _task.copyPublic)(userConfig);
  const webpackConfig = (0, _compile.getFinalWebpackConfig)(userConfig);
  webpackConfig.plugins.push(getAnalzerPlugin(port));
  return (0, _webpack().default)(webpackConfig, (err, stat) => {
    if (err) {
      console.error('Error', err);
      return;
    }
    if (stat && stat.hasErrors()) {
      console.log(_chalk().default.red(stat.toString('minimal')));
    }
    if (stat && stat.hasWarnings()) {
      console.log(_chalk().default.yellow(stat.toString('minimal')));
    }
    console.log(_chalk().default.bgGreen(_chalk().default.black(' DONE ')) + _chalk().default.green(' success !'));
  });
};