"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
function _path() {
  const data = _interopRequireDefault(require("path"));
  _path = function _path() {
    return data;
  };
  return data;
}
var _task = require("../task");
var _log = require("../log");
var _getUserConfig = _interopRequireDefault(require("../common/getUserConfig"));
var _loadDotEnv = _interopRequireDefault(require("../common/loadDotEnv"));
var _BabelRegister = _interopRequireDefault(require("../utils/BabelRegister/BabelRegister"));
function _webpack() {
  const data = _interopRequireDefault(require("webpack"));
  _webpack = function _webpack() {
    return data;
  };
  return data;
}
var _compile = require("../utils/compile");
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function _chalk() {
    return data;
  };
  return data;
}
var _utils = require("../utils");
var _watch = require("../watch");
function fs() {
  const data = _interopRequireWildcard(require("node:fs"));
  fs = function fs() {
    return data;
  };
  return data;
}
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && Object.prototype.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
const cwd = process.cwd();
const basePath = _path().default.join(cwd, '.env');
const localPath = `${basePath}.local`;
(0, _loadDotEnv.default)(basePath);
(0, _loadDotEnv.default)(localPath);
(0, _log.log)('register babel before config parsing');
const babelRegister = new _BabelRegister.default();
babelRegister.register();
function _default({
  watch,
  env = 'development',
  cache = true,
  callback
}) {
  const userConfig = (0, _getUserConfig.default)({
    cwd,
    env
  });
  startWatchDev(watch, userConfig, cache, callback);
  watch && startWatchLess(userConfig);
}
function cpLess(userConfig, callback) {
  if (!userConfig.useLess) {
    callback();
    return;
  }
  if (userConfig.entryStyle) {
    (0, _compile.compileLess2)(userConfig, callback);
  } else {
    (0, _compile.compileLess)(userConfig, callback);
  }
}
function startWatchDev(watch = true, userConfig, cache = true, callback = () => {}) {
  const finalWebpackConfig = (0, _compile.getFinalWebpackConfig)(userConfig, cache);
  return (0, _webpack().default)(_objectSpread(_objectSpread({}, finalWebpackConfig), {}, {
    watch
  }), (err, stats) => {
    if (err) {
      (0, _log.error)(err.stack || err);
      return;
    }
    const statConfig = _objectSpread({
      all: false,
      version: true,
      timings: true,
      modules: false,
      modulesSpace: 0,
      assets: false,
      assetsSpace: 0,
      errors: true,
      errorsCount: true,
      warnings: true,
      warningsCount: true,
      logging: 'warn'
    }, userConfig.stats);
    if (stats.hasErrors() || stats.hasWarnings()) {
      (0, _log.error)(stats.toString(statConfig));
    }
    if (userConfig.reportProfile) {
      if (userConfig.reportProfile.console) {
        console.log(stats.toString(statConfig));
      }
      if (userConfig.reportProfile.file) {
        // @ts-ignore
        fs().writeFileSync('build-stat.json', JSON.stringify(stats.toJson(statConfig)));
      }
    }
    (0, _task.copyPublic)(userConfig);
    // compile less
    cpLess(userConfig, () => {
      console.log(_chalk().default.bgGreen(_chalk().default.black(' DONE ')) + _chalk().default.green(' success !'));
      callback === null || callback === void 0 ? void 0 : callback();
    });
  });
}
function startWatchLess(userConfig) {
  if (!userConfig.useLess) {
    return;
  }
  let isRunning = false;
  let changed = false;
  function loop() {
    changed = false;
    cpLess(userConfig, () => {
      if (changed) {
        loop();
        return;
      }
      console.log(_chalk().default.bgGreen(_chalk().default.black(' Less ')) + _chalk().default.green(' success !'));
      isRunning = false;
    });
  }
  function startWatch() {
    if (process.env.WATCH !== 'none') {
      const ROUTER_CONFIG = 'ROUTER_CONFIG';
      const allPath = _path().default.join(cwd, 'src/**/*.less');
      const absPagesGlobPaths = [allPath].map(path => (0, _utils.winPath)(path));
      absPagesGlobPaths.forEach(path => {
        (0, _log.log)('\n' + path);
      });
      const watcher = (0, _watch.watch)(ROUTER_CONFIG, absPagesGlobPaths);
      watcher.on('error', event => (0, _log.log)(`\n${event}`)).on('all', (event, path) => {
        _utils.printLog.info(`[${event}] File ${path} was changed`);
        if (isRunning) {
          changed = true;
          return;
        }
        isRunning = true;
        loop();
      });
      process.on('SIGINT', () => {
        watcher.close();
        (0, _watch.unwatch)(ROUTER_CONFIG);
        process.exit(0);
      });
    }
  }
  startWatch();
}