"use strict";

function _path() {
  const data = _interopRequireDefault(require("path"));
  _path = function _path() {
    return data;
  };
  return data;
}
var _task = require("../task");
var _log = require("../log");
var _getUserConfig = _interopRequireDefault(require("../common/getUserConfig"));
var _loadDotEnv = _interopRequireDefault(require("../common/loadDotEnv"));
var _BabelRegister = _interopRequireDefault(require("../utils/BabelRegister/BabelRegister"));
function _webpack() {
  const data = _interopRequireDefault(require("webpack"));
  _webpack = function _webpack() {
    return data;
  };
  return data;
}
var _compile = require("../utils/compile");
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function _chalk() {
    return data;
  };
  return data;
}
function _webpackDevServer() {
  const data = _interopRequireDefault(require("webpack-dev-server"));
  _webpackDevServer = function _webpackDevServer() {
    return data;
  };
  return data;
}
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
const cwd = process.cwd();
const basePath = _path().default.join(cwd, '.env');
const localPath = `${basePath}.local`;
(0, _loadDotEnv.default)(basePath);
(0, _loadDotEnv.default)(localPath);
(0, _log.log)('register babel before config parsing');
const babelRegister = new _BabelRegister.default();
babelRegister.register();
const userConfig = (0, _getUserConfig.default)({
  cwd,
  env: 'development'
});
(0, _task.copyPublic)(userConfig);
startDevServer();
function getCORSConfig(cors) {
  return _objectSpread({
    origin: '*',
    credentials: true,
    methods: ['GET', 'POST', 'OPTIONS', 'PUT', 'DELETE', 'HEAD'],
    headers: ['DNT', 'Keep-Alive', 'User-Agent', 'Content-Language', 'If-Modified-Since', 'Cache-Control', 'Content-Type', 'Content-Range', 'Range'],
    expose: ['Content-Language', 'Cache-Control', 'Content-Type', 'Content-Length', 'Content-Range', 'Range']
  }, cors);
}
function startDevServer() {
  const finalWebpackConfig = (0, _compile.getFinalWebpackConfig)(userConfig);
  const port = userConfig.port || 9000;
  const before = userConfig.before,
    proxy = userConfig.proxy,
    scene = userConfig.scene;
  const entryOrigin = finalWebpackConfig.entry;
  Object.keys(entryOrigin).forEach(k => {
    entryOrigin[k] = [entryOrigin[k]];
  });
  const corsConfig = getCORSConfig(userConfig.cors);
  const compiler = (0, _webpack().default)(finalWebpackConfig);
  const pageUrl = `http://localhost:${port}/${scene && /^[\w-]+$/.test(scene) ? `?init=${scene}` : ''}`;
  const server = new (_webpackDevServer().default)({
    port,
    hot: true,
    historyApiFallback: true,
    open: pageUrl,
    static: [_path().default.resolve(cwd, userConfig.docker || './docker')],
    devMiddleware: {},
    setupMiddlewares: (middlewares, devServer) => {
      devServer.app.use((req, res, next) => {
        req.headers['X-Real-IP'] = req.ip;
        req.headers['X-Forwarded-For'] = req.ips.join(' ');
        res.set('Access-Control-Allow-Origin', corsConfig.origin);
        res.set('Access-Control-Allow-Credentials', String(corsConfig.credentials));
        res.set('Access-Control-Allow-Methods', corsConfig.methods.join(','));
        res.set('Access-Control-Allow-Headers', corsConfig.headers.join(','));
        res.set('Access-Control-Expose-Headers', corsConfig.expose.join(','));
        next();
      });
      if (before) {
        before(middlewares, devServer);
      }
      return middlewares;
    },
    proxy: _objectSpread({}, proxy),
    client: userConfig.client
  }, compiler);
  server.startCallback(err => {
    if (err) {
      console.error('ERROR:', err);
      return;
    }
    console.log(_chalk().default.cyan(`Project is running at ${pageUrl}`));
  });
}